from matrix import *
from math import cos,sin
from formes import *


def threeD_to_twoD(points,angles = (0,0,0),distance = 1.5,perspective = True):
  projected = []
  
  rotationX = (
    (1,0,0),
    (0,cos(angles[0]),-sin(angles[0])),
    (0,sin(angles[0]),cos(angles[0]))
  )
  rotationY = (
    (cos(angles[1]),0,-sin(angles[1])),
    (0,1,0),
    (sin(angles[1]),0,cos(angles[1]))
  )
  rotationZ = (
    (cos(angles[2]),-sin(angles[2]),0),
    (sin(angles[2]),cos(angles[2]),0),
    (0,0,1)
  )
  
  for point in points:
    rotated = multiply(rotationZ,vector_to_matrix(point))
    rotated = multiply(rotationY,rotated)
    rotated = multiply(rotationX,rotated)
    
    if perspective:
      z = 1/(distance-rotated[2][0])
      
      projection = (
        (z,0,0),
        (0,z,0),
        (0,0,0)
      )
    else:
      projection = (
        (1,0,0),
        (0,1,0),
        (0,0,0)
      )
    
    projected2d = matrix_to_vector(multiply(projection,rotated))
    projected2d[0] *= 100
    projected2d[1] *= -100
    projected2d[2] *= 100
    
    projected.append(projected2d)
  
  return projected

def fourD_to_threeD(points,angles = (0,0,0,0),distance = 1.5,perspective = True):
  rotationXY = (
    (cos(angles[2]),-sin(angles[2]),0,0),
    (sin(angles[2]),cos(angles[2]),0,0),
    (0,0,1,0),
    (0,0,0,1)
  )
  rotationXZ = (
    (cos(angles[1]),0,-sin(angles[1]),0),
    (0,1,0,0),
    (sin(angles[1]),0,cos(angles[1]),0),
    (0,0,0,1)
  )
  rotationYZ = (
    (1,0,0,0),
    (0,cos(angles[0]),-sin(angles[0]),0),
    (0,sin(angles[0]),cos(angles[0]),0),
    (0,0,0,1)
  )
  rotationZW = (
    (1,0,0,0),
    (0,1,0,0),
    (0,0,cos(angles[3]),-sin(angles[3])),
    (0,0,sin(angles[3]),cos(angles[3]))
  )
  
  projected = []
  
  for point in points:
    rotated = multiply(rotationXY,vector_to_matrix(point))
    rotated = multiply(rotationXZ,rotated)
    rotated = multiply(rotationYZ,rotated)
    rotated = multiply(rotationZW,rotated)
    
    if perspective:
      w = 1/(distance-point[3])
      
      projection = (
        (w,0,0,0),
        (0,w,0,0),
        (0,0,w,0)
      )
    else:
      projection = (
        (1,0,0,0),
        (0,1,0,0),
        (0,0,1,0)
      )
    
    projected3d = matrix_to_vector(multiply(projection,rotated))
    
    projected.append(projected3d)
  
  return projected

def draw_box(coords,sizes,angles = (0,0,0),fill = False,color = (0,0,0),distance = 1.5,perspective = True):
  if fill:
    for z in range(coords[2]*100,(coords[2]+sizes[2])*100):
      for y in range(coords[1]*100,(coords[1]+sizes[1])*100):
        for x in range(coords[0]*100,(coords[0]+sizes[0])*100):
          projected = threeD_to_twoD([(x/100,y/100,z/100)])
          set_pixel(191+int(projected[0][0]),95+int(projected[0][1]),color)
          #points.append((x/100,y/100,z/100))
  else:
    points = []
    for z in [coords[2],coords[2]+sizes[2]]:
      points.append((coords[0],coords[1],z))
      points.append((coords[0]+sizes[0],coords[1],z))
      points.append((coords[0]+sizes[0],coords[1]+sizes[1],z))
      points.append((coords[0],coords[1]+sizes[1],z))
  
    projected = threeD_to_twoD(points,angles,distance,perspective)
    
    for i in range(4):
      connect(i,(i+1)%4,projected,color)
      connect(i+4,((i+5)%4)+4,projected,color)
      connect(i,i+4,projected,color)

def connect(i,j,points,color = (0,0,0)):
  a = points[i]
  b = points[j]
  
  line(191+int(a[0]),95+int(a[1]),191+int(b[0]),95+int(b[1]),color)
